/**
 *	XDataGrid Component
 *
 *	Copyright (C) 2004  Kamran Zafar
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version 2
 *	of the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
 * XDataGridXMLReader.java
 * Created on Sep 2, 2004
 */
package xeus.db.datagrid.xml.io;

import java.io.IOException;
import java.util.Vector;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

import xeus.db.datagrid.XDataSet;

/**
 * @author Kamran Zafar
 *  
 */
public class XDataSetXMLHandler extends DefaultHandler {

    private boolean isTableHeader;
    private boolean isTableBody;
    private boolean nextTable;
    private boolean nextCol;
    private boolean nextRow;
    private boolean root;

    private boolean flag;

    private XDataSet dataSet;
    private Vector columns;
    private Vector rows;
    private Vector row;

    private String element;

    public XDataSetXMLHandler() {
        isTableHeader = false;
        isTableBody = false;
        nextTable = false;
        nextCol = false;
        nextRow = false;
        root = false;
        flag = true;

        element = "";
        columns =null;
        rows =null;
        dataSet = new XDataSet();
    }

    public void read(String fileName) throws IOException,SAXException {
        XMLReader readerObj = XMLReaderFactory
                .createXMLReader("org.apache.xerces.parsers.SAXParser");
        
        readerObj.setFeature("http://xml.org/sax/features/validation", true);
        readerObj.setContentHandler(this);
        readerObj.setErrorHandler(this);
        readerObj.parse(fileName);
    }

    public void characters(char[] ch, int start, int length)
            throws SAXException {
        String data = new String(ch, start, length);

        if (isTableHeader && nextCol) {

            if (flag) {
                columns.add(data.trim());
                flag = false;
            } else
                flag = true;
        }

        if (isTableBody && nextCol && nextRow) {

            if (flag) {
                row.add(data.trim());
                flag = false;
            } else
                flag = true;
        }
    }

    public void endDocument() throws SAXException {

    }

    public void endElement(String uri, String localName, String qName)
            throws SAXException {

        if (qName.trim().equals("xdataset")) {
            if (root)
                root = true;
            else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtable")) {
            if (nextTable) {
                dataSet.addTable(rows, columns);
                rows=null;
                columns=null;
                nextTable = false;
            } else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtableheader")) {
            if (isTableHeader)
                isTableHeader = false;
            else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtablebody")) {
            if (isTableBody)
                isTableBody = false;
            else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtr")) {
            if (nextRow) {
                rows.add(row);
                nextRow = false;
            } else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtd")) {
            if (nextCol)
                nextCol = false;
            else
                throw new SAXException("Misplaced element: "+qName);
        }
    }

    public void startDocument() throws SAXException {
    }

    public void startElement(String uri, String localName, String qName,
            Attributes attr) throws SAXException {

        if (qName.trim().equals("xdataset")) {
            if (!root && !nextTable && !nextCol && !isTableBody
                    && !isTableHeader && !nextRow) {
                element = qName;
                root = true;
            } else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtable")) {
            if (root && !nextTable && !nextCol && !isTableBody
                    && !isTableHeader && !nextRow) {
                element = qName;
                rows=new Vector();
                columns=new Vector();
                nextTable = true;
            } else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtableheader")) {
            if (root && nextTable && !nextCol && !isTableBody && !isTableHeader
                    && !nextRow) {
                element = qName;
                isTableHeader = true;
            } else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtablebody")) {
            if (root && nextTable && !nextCol && !isTableBody && !isTableHeader
                    && !nextRow) {
                element = qName;
                isTableBody = true;
            } else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtr")) {
            if (root && nextTable && !nextCol && isTableBody && !isTableHeader
                    && !nextRow) {
                row = new Vector();
                nextRow = true;
                element = qName;
            } else
                throw new SAXException("Misplaced element: "+qName);
        }

        if (qName.trim().equals("xtd")) {
            if (root && nextTable && !nextCol && !isTableBody && isTableHeader
                    && !nextRow) {
                element = qName;
                nextCol = true;
            } else if (root && nextTable && !nextCol && isTableBody
                    && !isTableHeader && nextRow) {
                element = qName;
                nextCol = true;
            } else
                throw new SAXException("Misplaced element: "+qName);
        }
    }

    public XDataSet getXDataSet() {
        return dataSet;
    }

    public void warning(SAXParseException exception) throws SAXException {
        System.err.println("Warning in " + exception.getSystemId()
                + " at line " + exception.getLineNumber() + ", column "
                + exception.getColumnNumber());
    }

    public void error(SAXParseException exception) throws SAXException {
        System.err.println("Error in " + exception.getSystemId() + " at line "
                + exception.getLineNumber() + ", column "
                + exception.getColumnNumber());
    }

    public void fatalError(SAXParseException exception) throws SAXException {
        System.err.println("Fatal error in " + exception.getSystemId()
                + " at line " + exception.getLineNumber() + ", column "
                + exception.getColumnNumber());
        throw exception;
    }
}